@@ -1,35 +1,78 @@
 """A lightweight book theme based on the pydata sphinx theme."""
+import os
 from pathlib import Path
+
+try:
+    import importlib.resources as resources
+except ImportError:
+    # python < 3.7
+    import importlib_resources as resources
+
+from bs4 import BeautifulSoup as bs
 from docutils.parsers.rst import directives
 from docutils import nodes
+from sphinx.application import Sphinx
+from sphinx.locale import get_translation
 from sphinx.util import logging
-from bs4 import BeautifulSoup as bs
-from sass import compile as sass_compile
 
 from .launch import add_hub_urls
+from . import static as theme_static
 
-__version__ = "0.0.36"
+__version__ = "0.0.37"
 """sphinx-book-theme version"""
 
 SPHINX_LOGGER = logging.getLogger(__name__)
+MESSAGE_CATALOG_NAME = "booktheme"
 
 
 def get_html_theme_path():
     """Return list of HTML theme paths."""
-    theme_path = str(Path(__file__).parent.absolute())
+    theme_path = os.path.abspath(Path(__file__).parent)
     return theme_path
 
 
-def add_static_path(app):
-    static_path = Path(__file__).parent.joinpath("static").absolute()
-    app.config.html_static_path.append(str(static_path))
+def add_static_paths(app):
+    """Ensure CSS/JS is loaded."""
+    app.env.book_theme_resources_changed = False
 
-    # Compile the css file if it's not been compiled already
-    compiled_css_file = static_path / "sphinx-book-theme.css"
-    if not compiled_css_file.exists():
-        source_dir = str(static_path.parent / "scss")
-        output_dir = str(static_path)
-        sass_compile(dirname=(source_dir, output_dir), output_style="compressed")
+    output_static_folder = Path(app.outdir) / "_static"
+    theme_static_files = resources.contents(theme_static)
+
+    if (
+        app.config.html_theme_options.get("theme_dev_mode", False)
+        and output_static_folder.exists()
+    ):
+        # during development, the JS/CSS may change, if this is the case,
+        # we want to remove the old files and ensure that the new files are loaded
+        for path in output_static_folder.glob("sphinx-book-theme*"):
+            if path.name not in theme_static_files:
+                app.env.book_theme_resources_changed = True
+                path.unlink()
+        # note sphinx treats theme css different to regular css
+        # (it is specified in theme.conf), so we don't directly use app.add_css_file
+        for fname in resources.contents(theme_static):
+            if fname.endswith(".css"):
+                if not (output_static_folder / fname).exists():
+                    (output_static_folder / fname).write_bytes(
+                        resources.read_binary(theme_static, fname)
+                    )
+                    app.env.book_theme_resources_changed = True
+
+    # add javascript
+    for fname in resources.contents(theme_static):
+        if fname.endswith(".js"):
+            app.add_js_file(fname)
+
+
+def update_all(app, env):
+    """During development, if CSS/JS has changed, all files should be re-written,
+    to load the correct resources.
+    """
+    if (
+        app.config.html_theme_options.get("theme_dev_mode", False)
+        and env.book_theme_resources_changed
+    ):
+        return list(env.all_docs.keys())
 
 
 def find_url_relative_to_root(pagename, relative_page, path_docs_source):
@@ -49,7 +92,7 @@ def find_url_relative_to_root(pagename, relative_page, path_docs_source):
     path_rel_from_page_dir = source_dir.joinpath(
         path_parent.parent.joinpath(path_rel.parent)
     )
-    path_from_page_dir = path_rel_from_page_dir.resolve()
+    path_from_page_dir = Path(os.path.abspath(path_rel_from_page_dir))
     page_rel_root = path_from_page_dir.relative_to(source_dir).joinpath(path_rel.name)
     return page_rel_root
 
@@ -82,7 +125,10 @@ def add_to_context(app, pagename, templatename, context, doctree):
             master_doc = config["master_doc"]
             master_doctree = app.env.get_doctree(master_doc)
             master_url = context["pathto"](master_doc)
-            master_title = list(master_doctree.traverse(nodes.title))[0].astext()
+            master_title = list(master_doctree.traverse(nodes.title))
+            if len(master_title) == 0:
+                raise ValueError(f"Landing page missing a title: {master_doc}")
+            master_title = master_title[0].astext()
 
             # Insert it into our toctree
             ul_home = bs(
@@ -130,10 +176,11 @@ def add_to_context(app, pagename, templatename, context, doctree):
     def generate_toc_html():
         """Return the within-page TOC links in HTML."""
 
-        if not context.get("toc"):
+        toc = context.get("toc")
+        if not toc:
             return ""
 
-        soup = bs(context["toc"], "html.parser")
+        soup = bs(toc, "html.parser")
 
         # Add toc-hN classes
         def add_header_level_recursive(ul, level):
@@ -154,17 +201,28 @@ def add_to_context(app, pagename, templatename, context, doctree):
                 a = li.find("a")
                 a["class"] = a.get("class", []) + ["nav-link"]
 
-        # Keep only the sub-sections of the title (so no title is shown)
-        title = soup.find("a", attrs={"href": "#"})
-        if title:
-            title = title.parent
-            # Only show if children of the title item exist
-            if title.select("ul li"):
-                out = title.find("ul").prettify()
-            else:
-                out = ""
+        # If we only have one h1 header, assume it's a title
+        h1_headers = soup.select(".toc-h1")
+        if len(h1_headers) == 1:
+            title = h1_headers[0]
+            # If we have no sub-headers of a title then we won't have a TOC
+            if not title.select(".toc-h2"):
+                return ""
+
+            toc_out = title.find("ul").prettify()
+
+        # Else treat the h1 headers as sections
         else:
-            out = ""
+            toc_out = soup.prettify()
+
+        out = f"""
+        <div class="tocsection onthispage pt-5 pb-3">
+            <i class="fas fa-list"></i> { context["translate"]('Contents') }
+        </div>
+        <nav id="bd-toc-nav">
+            {toc_out}
+        </nav>
+        """
         return out
 
     context["generate_toc_html"] = generate_toc_html
@@ -228,6 +286,13 @@ def add_to_context(app, pagename, templatename, context, doctree):
         if key in context:
             context[key] = _string_or_bool(context[key])
 
+    translation = get_translation(MESSAGE_CATALOG_NAME)
+    context["translate"] = translation
+    # this is set in the html_theme
+    context["theme_search_bar_text"] = translation(
+        context.get("theme_search_bar_text", "Search the docs ...")
+    )
+
 
 def update_thebe_config(app, env, docnames):
     """Update thebe configuration with SBT-specific values"""
@@ -303,16 +368,21 @@ class Margin(directives.body.Sidebar):
         return nodes
 
 
-def setup(app):
+def setup(app: Sphinx):
     app.connect("env-before-read-docs", update_thebe_config)
 
     # Configuration for Juypter Book
     app.connect("html-page-context", add_hub_urls)
 
-    app.connect("builder-inited", add_static_path)
+    app.connect("builder-inited", add_static_paths)
+    app.connect("env-updated", update_all)
+
+    # add translations
+    package_dir = os.path.abspath(os.path.dirname(__file__))
+    locale_dir = os.path.join(package_dir, "translations", "locales")
+    app.add_message_catalog(MESSAGE_CATALOG_NAME, locale_dir)
 
     app.add_html_theme("sphinx_book_theme", get_html_theme_path())
     app.connect("html-page-context", add_to_context)
 
-    app.add_js_file("sphinx-book-theme.js")
     app.add_directive("margin", Margin)
