@@ -1,4 +1,8 @@
 from pathlib import Path
+from typing import Any, Dict, Optional
+
+from docutils.nodes import document
+from sphinx.application import Sphinx
 from sphinx.util import logging
 from shutil import copy2
 
@@ -6,8 +10,24 @@ from shutil import copy2
 SPHINX_LOGGER = logging.getLogger(__name__)
 
 
-def add_hub_urls(app, pagename, templatename, context, doctree):
-    """Builds a binder link and inserts it in HTML context for use in templating."""
+def add_hub_urls(
+    app: Sphinx,
+    pagename: str,
+    templatename: str,
+    context: Dict[str, Any],
+    doctree: Optional[document],
+):
+    """Builds a binder link and inserts it in HTML context for use in templating.
+
+    This is a ``html-page-context`` sphinx event (see :ref:`sphinx:events`).
+
+    :param pagename: The sphinx docname related to the page
+    :param context: A dictionary of values that are given to the template engine,
+        to render the page and can be modified to include custom values.
+    :param doctree: A doctree when the page is created from a reST documents;
+        it will be None when the page is created from an HTML template alone.
+
+    """
 
     # First decide if we'll insert any links
     path = app.env.doc2path(pagename)
@@ -29,7 +49,7 @@ def add_hub_urls(app, pagename, templatename, context, doctree):
         path_ntbk = ntbk_dir.joinpath(pagename).with_suffix(".ipynb")
         path_new_notebook = sources_dir.joinpath(pagename).with_suffix(".ipynb")
         # Copy the notebook to `_sources` dir so it can be downloaded
-        path_new_notebook.parent.mkdir(exist_ok=True)
+        path_new_notebook.parent.mkdir(exist_ok=True, parents=True)
         copy2(path_ntbk, path_new_notebook)
         context["ipynb_source"] = pagename + ".ipynb"
 
@@ -81,7 +101,7 @@ def add_hub_urls(app, pagename, templatename, context, doctree):
     if jupyterhub_url:
         url = (
             f"{jupyterhub_url}/hub/user-redirect/git-pull?"
-            f"repo={repo_url}&urlpath={ui_pre}/{repo}/{path_rel_repo}"
+            f"repo={repo_url}&urlpath={ui_pre}/{repo}/{path_rel_repo}&branch={branch}"
         )
         context["jupyterhub_url"] = url
 
